@@ -348,7 +348,7 @@ int do_bedbug_stack(struct cmd_tbl *cmdtp, int flag, int argc,
 		return 1;
 	}
 
-	top = gd->bd->bi_memstart + gd->bd->bi_memsize;
+	top = gd->ram_start + gd->ram_size;
 	depth = 0;
 
 	printf ("Depth     PC\n");
